SystemOrganization addCategory: #'Mondrian-Shapes'!

MOShape subclass: #MOLineShape
	instanceVariableNames: 'color width fromPositions toPositions'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOLineShape commentStamp: 'Alexandre.Bergel 8/30/2009 13:59' prior: 0!
Maybe fromPositions and toPositions are positioned to high in the hierarchy. 
Wondering whether these variables may be safely moved down into MOStraightLineShape

A MOLineShape is xxxxxxxxx.

Instance Variables
	color:		<Object>
	fromPositions:		<Object>
	toPositions:		<Object>
	width:		<Object>

color
	- xxxxx

fromPositions
	- xxxxx

toPositions
	- xxxxx

width
	- xxxxx
!

MOLineShape subclass: #MOCurveFitterShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOCurveFitterShape commentStamp: '<historical>' prior: 0!
MOCurveFitterShape represents a shape describing a conic section determined by three points p1, p2 and p3. I interpolate p1 and p3 and am tangent to line p1, p2 at p1 and line p3, p2 at p3. 

Class collaborations
	MOGraphElement MOEdge 

Class public API
		#display:on:	#lineSegmentsFor:

Testing classes
	MOViewRendererTest 

Comments written by AlexandreBergel on 28 April 2011
!

!MOCurveFitterShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:13'!
acceptVisitor: aVisitor
^aVisitor visitCurveFitterShape: self.! !

!MOCurveFitterShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 14:22'!
lineSegmentsFor: anEdgeElement
	"return instances of LineSegment that describes the segment of myself. Coordinate are relatives to the parent object (owner)"

	| yUnit toPoint fromPoint running path tmp t |
	t := self getAttachPointsFor: anEdgeElement.
	fromPoint := t first.
	toPoint := t second.

	yUnit := ((toPoint - fromPoint) y / 3) asInteger. 

	running := fromPoint.
	path := Array
				with: ((0 @ yUnit) + fromPoint)
				with: toPoint.
	^ path collect: [:p | tmp := LineSegment from: running to: p. running := p. tmp]
! !

!MOLineShape classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 3/29/2009 15:38'!
color: aColor
	^ self new color: aColor; yourself! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:45'!
absoluteBoundsFor: edgeElement
	"Answer the bounds in absolute terms (relative to the entire Canvas, not just the parent)."

	^ Rectangle 
			encompassing: { edgeElement source absoluteBounds origin . 
							  edgeElement source absoluteBounds corner . 
							  edgeElement target absoluteBounds origin .
							   edgeElement target absoluteBounds origin
							}  
! !

!MOLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/28/2011 23:46'!
color
	"Answer the color of the line"

	^ color! !

!MOLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/28/2011 23:47'!
color: aBlockASymbolOrAColor
	"Set the color of the line."

	color := aBlockASymbolOrAColor! !

!MOLineShape methodsFor: 'private' stamp: 'AlexandreBergel 4/28/2011 23:47'!
colorFor: anElement
	"Answer the color of anElement"

	^ anElement 
		cacheNamed: #cachecolorFor:
		of: self
		ifAbsentInitializeWith: [ self computeColorFor: anElement ]

! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:47'!
computeAttachPointsFor: fromNode and: toNode
	"Return the two attach points according to their anchor"

	| fromPoint toPoint b1 b2 |
	b1 := fromNode absoluteBounds.
	b2 := toNode absoluteBounds.
	((fromPositions isNil not and: [ toPositions isNil not ])
		and: [ fromPositions isEmpty not and: [ toPositions isEmpty not ] ])
		ifTrue:
			[ 
			fromPositions first == #center
				ifTrue: [ fromPoint := b1 center ].
			fromPositions first == #leftCenter
				ifTrue: [ fromPoint := b1 leftCenter ].
			fromPositions first == #rightCenter
				ifTrue: [ fromPoint := b1 rightCenter ].
			fromPositions first == #topRight
				ifTrue: [ fromPoint := b1 topRight ].
			fromPositions first == #bottomRight
				ifTrue: [ fromPoint := b1 bottomRight ].
			fromPositions first == #bottomLeft
				ifTrue: [ fromPoint := b1 bottomLeft ].
			fromPositions first == #bottomCenter
				ifTrue: [ fromPoint := b1 bottomCenter ].
			fromPositions first == #topCenter
				ifTrue: [ fromPoint := b1 topCenter ].
			toPositions first == #center
				ifTrue: [ toPoint := b2 center ].
			toPositions first == #leftCenter
				ifTrue: [ toPoint := b2 leftCenter ].
			toPositions first == #rightCenter
				ifTrue: [ toPoint := b2 rightCenter ].
			toPositions first == #topRight
				ifTrue: [ toPoint := b2 topRight ].
			toPositions first == #bottomRight
				ifTrue: [ toPoint := b2 bottomRight ].
			toPositions first == #bottomCenter
				ifTrue: [ toPoint := b2 bottomCenter ].
			toPositions first == #topCenter
				ifTrue: [ toPoint := b2 topCenter ].
			^ {fromPoint.
			toPoint} ].
	fromPoint := nil.
	toPoint := nil.
	(b1 right < b2 left and: [ b1 top > b2 bottom ])
		ifTrue:
			[ fromPoint := b1 topCenter.
			toPoint := b2 bottomCenter ].
	(b1 right < b2 left and: [ b1 bottom <= b2 top ])
		ifTrue:
			[ fromPoint := b1 bottomCenter.
			toPoint := b2 topCenter ].
	(b2 right < b1 left and: [ b2 top > b1 bottom ])
		ifTrue:
			[ fromPoint := b2 topCenter.
			toPoint := b1 bottomCenter ].
	(b2 right < b1 left and: [ b2 bottom <= b1 top ])
		ifTrue:
			[ fromPoint := b1 topCenter.
			toPoint := b2 bottomCenter ].	"---"
	(b1 right <= b2 left and: [ b1 top between: b2 top and: b2 bottom ])
		ifTrue:
			[ fromPoint := b1 rightCenter.
			toPoint := b2 leftCenter ].
	(b2 right < b1 left and: [ b2 top > b1 top and: [ b2 top > b1 bottom ] ])
		ifTrue:
			[ fromPoint := b1 leftCenter.
			toPoint := b2 rightCenter ].
	(b1 left >= b2 right and: [ b2 top between: b1 top and: b1 bottom ])
		ifTrue:
			[ fromPoint := b1 leftCenter.
			toPoint := b2 rightCenter ].
	(b1 left > b2 right and: [ b1 top > b2 top and: [ b1 top < b2 bottom ] ])
		ifTrue:
			[ fromPoint := b1 leftCenter.
			toPoint := b2 rightCenter ].
	(b1 right < b2 left and: [ b1 top < b2 top and: [ b1 bottom > b2 top ] ])
		ifTrue:
			[ fromPoint := b1 rightCenter.
			toPoint := b2 leftCenter ].
	(b1 top >= b2 bottom and: [ b1 left between: b2 left and: b2 right ])
		ifTrue:
			[ fromPoint := b1 topCenter.
			toPoint := b2 bottomCenter ].
	(b1 top > b2 bottom and: [ b1 right > b2 left and: [ b1 right < b2 right ] ])
		ifTrue:
			[ fromPoint := b1 topCenter.
			toPoint := b2 bottomCenter ].
	fromPoint ifNil: [ fromPoint := b1 bottomCenter ].
	toPoint ifNil: [ toPoint := b2 topCenter ].

	^ {fromPoint . toPoint}! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:48'!
computeBoundsFor: anEdge
	"Answer the bounds of anEdge"
	
	| t |
	t := self getAttachPointsFor: anEdge.
	^ Rectangle encompassing: t

! !

!MOLineShape methodsFor: 'private' stamp: 'AlexandreBergel 4/28/2011 23:48'!
computeColorFor: anElement
	"Answer the color of anElement"

	^ color moValue: anElement model! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:48'!
computeExtentFor: anElement 
	"Answer the extend of anElement"

	^ (anElement source bounds bottomCenter corner:
		anElement target bounds topCenter) extent! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:48'!
computeWidthFor: anElement
	"Answer the with of anElement"

	^ (width moValue: anElement model) asInteger! !

!MOLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 23:49'!
containsPoint: aPoint for: anElement
	"True if aPoint is contained in the line anElement"

	self lineSegmentsFor: anElement  do:
		[:p1 :p2 |
		(aPoint onLineFrom: p1 to: p2 within: (3 max: (self widthFor: anElement)+1//2) asFloat)
				ifTrue: [^ true]].
	^ false! !

!MOLineShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/28/2011 23:49'!
defaultColor
	"Default color for lines"

	^ Color lightGray! !

!MOLineShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/28/2011 23:49'!
defaultWidth
	"Default width for lines"
	
	^ 1! !

!MOLineShape methodsFor: 'accessing' stamp: 'Alexandre.Bergel 4/12/2009 12:14'!
fromPositions
	^ fromPositions! !

!MOLineShape methodsFor: 'accessing' stamp: 'Alexandre.Bergel 4/12/2009 12:14'!
fromPositions: anObject
	fromPositions := anObject! !

!MOLineShape methodsFor: 'accessing-computed' stamp: 'Alexandre Bergel 7/17/2010 08:26'!
getAttachPointsFor: anEdge
	| fromPoint  toPoint t |
	(anEdge source isNotSelected
		and: [ anEdge target isNotSelected and: [ anEdge isCacheLoaded ] ])
			ifTrue: [ fromPoint := anEdge cacheFromPoint. toPoint := anEdge cacheToPoint]
			ifFalse: 
				[ t := self computeAttachPointsFor: anEdge source and: anEdge target.
				fromPoint := t first.
				toPoint := t second.
				anEdge cacheFromPoint: fromPoint.
				anEdge cacheToPoint: toPoint ].
	^ {fromPoint . toPoint}
! !

!MOLineShape methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 4/28/2011 23:50'!
heightFor: anElement
	"We cancel this method since it does not make sense"
	
	self error: 'Not appropriate'! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 10/4/2011 08:49'!
if: conditionBlock color: colorBlock
	"If conditionBlock is evaluated at true, then colorBlock is used to set the color of the line. Both conditionBlock and colorBlock are evaluated with the model value of the node."

	| oldBlockOrValue |
	oldBlockOrValue := self color ifNil: [ self defaultColor ].
	^self color: [ :each | (conditionBlock moValue: each)
								ifTrue: [ colorBlock moValue: each ] ifFalse: [ oldBlockOrValue moValue: each ]].! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 10/4/2011 08:51'!
if: conditionBlock width: widthBlock
	"If conditionBlock is evaluated at true, then widthBlock is used to set the width of the line. Both conditionBlock and widthBlock are evaluated with the model value of the edge."

	| oldBlockOrValue |
	oldBlockOrValue := self width ifNil: [ self defaultWidth ].
	^self width: [ :each | (conditionBlock moValue: each)
								ifTrue: [ widthBlock moValue: each ] ifFalse: [ oldBlockOrValue moValue: each ]].! !

!MOLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/28/2011 23:50'!
initialize
	"Initialization"

	super initialize.
	self color: self defaultColor.
	self width: self defaultWidth.
! !

!MOLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 23:50'!
isArrowed
	"True if I am arrowed. Overidden in several subclasses"

	^ false! !

!MOLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 23:50'!
isDashed
	"True if I am dashed. Overidden in several subclasses"

	^ false! !

!MOLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 23:51'!
isLineShape
	"True if I am a line shape. This method is intended to be overriden."

	^ true! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 23:51'!
lineSegmentsFor: anEdgeElement
	"return instances of LineSegment that describes the segment of myself. Coordinate are relatives to the parent object (owner)"
	
	| attach |
	attach := self getAttachPointsFor: anEdgeElement.
	^ { LineSegment from: (attach first) to: (attach second) }
! !

!MOLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/28/2011 23:51'!
lineSegmentsFor: anElement  do: aTwoArgBlock
	"Iteration over the segments"

	(self lineSegmentsFor: anElement)
		do: [:seg | seg lineSegmentsDo: aTwoArgBlock ]! !

!MOLineShape methodsFor: 'accessing' stamp: 'Alexandre.Bergel 4/12/2009 12:15'!
toPositions
	^ toPositions! !

!MOLineShape methodsFor: 'accessing' stamp: 'Alexandre.Bergel 4/12/2009 12:15'!
toPositions: anObject
	toPositions := anObject! !

!MOLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/28/2011 18:47'!
updateBounds: anElement for: aBounds
	self error: 'Not appropriate'! !

!MOLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/28/2011 23:52'!
width
	"Answer the width of the line. width could be a block, a symbol or a value"

	^ width! !

!MOLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/28/2011 23:52'!
width: anObject
	"The width of the line. width could be a block, a symbol or a value"

	width := anObject! !

!MOLineShape methodsFor: 'accessing-computed' stamp: 'AlexandreBergel 4/28/2011 23:52'!
widthFor: anElement
	"Answer the width of the line anElement"

	^ self isCached 
		ifTrue: [ anElement cacheNamed: #cachewidthFor: of: self  ifAbsentInitializeWith: [ self computeWidthFor: anElement ] ]
		ifFalse: [ self computeWidthFor: anElement ].

! !

MOLineShape subclass: #MOOrthoHorizontalLineShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOOrthoHorizontalLineShape commentStamp: '<historical>' prior: 0!
MOOrthoHorizontalLineShape represents a line with two 90 degrees corners.  The line is horizontally oriented on the extremities. 

Class collaborations
	MOEdge MOGraphElement 

Class public API
		#display:on:

Testing classes
	 MOViewRendererTest

Comments written by AlexandreBergel on 28 April 2011
!

MOOrthoHorizontalLineShape subclass: #MOArrowedOrthoHorizontalLineShape
	instanceVariableNames: 'arrowSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOArrowedOrthoHorizontalLineShape commentStamp: '<historical>' prior: 0!
MOArrowedOrthoHorizontalLineShape is similar to MOOrthoHorizontalLineShape, except that it is arrowed on its extremity

Class collaborations
	MOEasel 

Class public API
		#arrowSize	#display:on:

Testing classes
	MOViewRendererTest 

Instance Variables:
	arrowSize --  Size of the arrow, an integer.

Comments written by AlexandreBergel on 28 April 2011
!

!MOArrowedOrthoHorizontalLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:12'!
acceptVisitor: aVisitor
^aVisitor visitArrowedOrthoHorizontalLineShape: self.! !

!MOArrowedOrthoHorizontalLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize
	"Return the size of the arrow, an integer"

	^ arrowSize! !

!MOArrowedOrthoHorizontalLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize: anInteger
	"Set the size of the arrow, an integer"

	arrowSize := anInteger! !

!MOArrowedOrthoHorizontalLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 16:56'!
defaultArrowSize
	"Answer the default arrow size"

	^ 6! !

!MOArrowedOrthoHorizontalLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 11:19'!
initialize
	"initialization"

	super initialize.
	self arrowSize: self defaultArrowSize! !

!MOOrthoHorizontalLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:15'!
acceptVisitor: aVisitor
^aVisitor visitOrthoHorizontalLineShape: self.! !

MOLineShape subclass: #MOOrthoVerticalLineShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOOrthoVerticalLineShape commentStamp: '<historical>' prior: 0!
MOOrthoVerticalLineShape represents a line with two 90 degrees corners.  The line is vertically oriented on the extremities.

Class collaborations
	MOEdge MOGraphElement 

Class public API
		#display:on:	#lineSegmentsFor:

Testing classes
	MOViewRendererTest 

Comments written by AlexandreBergel on 28 April 2011
!

MOOrthoVerticalLineShape subclass: #MOArrowedOrthoVerticalLineShape
	instanceVariableNames: 'arrowSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOArrowedOrthoVerticalLineShape commentStamp: '<historical>' prior: 0!
MOArrowedOrthoVerticalLineShape is similar to MOOrthoVerticalLineShape but has an arrow on the destination node.

Class collaborations
	MOEasel 

Class public API
		#arrowSize	#display:on:

Testing classes
	MOViewRendererTest 

Instance Variables:
	arrowSize --  size of the arrow

Comments written by AlexandreBergel on 28 April 2011
!

!MOArrowedOrthoVerticalLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:13'!
acceptVisitor: aVisitor
^aVisitor visitArrowedOrthoVerticalLineShape: self.! !

!MOArrowedOrthoVerticalLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize
	"Return the size of the arrow, an integer"

	^ arrowSize! !

!MOArrowedOrthoVerticalLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:21'!
arrowSize: anInteger
	"Set the size of the arrow, an integer"

	arrowSize := anInteger! !

!MOArrowedOrthoVerticalLineShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 16:58'!
defaultArrowSize
	"Answer the default arrow size"

	^ 6! !

!MOArrowedOrthoVerticalLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 11:22'!
initialize
	"initialization"

	super initialize.
	self arrowSize: self defaultArrowSize! !

!MOOrthoVerticalLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:16'!
acceptVisitor: aVisitor
^aVisitor visitOrthoVerticalLineShape: self.! !

!MOOrthoVerticalLineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 11:19'!
lineSegmentsFor: anEdgeElement
	"Return the list of segments"

	| running path tmp fromNode toNode fromPoint toPoint edgeBound midy |
	
	fromNode := anEdgeElement source.
	toNode := anEdgeElement target.
	fromPoint := fromNode bounds bottomCenter.
	toPoint := toNode bounds topCenter.

	edgeBound := fromPoint corner: toPoint.
	
	midy := ((edgeBound corner y - edgeBound origin y) / 2) asInteger + edgeBound origin y.

	running := fromPoint.
	path := Array
				with: (edgeBound origin x @ midy)
				with: (edgeBound corner x @ midy)
				with: toPoint.
	^ path collect: [:p | tmp := LineSegment from: running to: p. running := p. tmp]! !

MOLineShape subclass: #MOSplineShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOSplineShape commentStamp: '<historical>' prior: 0!
MOSplineShape is a shape based on cubic spline curve. Note that this shape is computationally expensive. Do not abuse them else your image may be stuck in a the render loop.

Class collaborations
	MOGraphElement MOEdge 

Class public API
		#display:on:	#lineSegmentsFor:

Testing classes
	MOViewRendererTest 

Comments written by AlexandreBergel on 28 April 2011
!

!MOSplineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:20'!
acceptVisitor: aVisitor
^aVisitor visitSplineShape: self.! !

!MOSplineShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 11:24'!
lineSegmentsFor: anElement
	"return instances of LineSegment that describes the segment of myself. Coordinate are relatives to the parent object (owner)"

	| t fromPoint toPoint xUnit yUnit running path tmp |
	
	t := self getAttachPointsFor: anElement.
	fromPoint := t first.
	toPoint := t second.

	xUnit := ((toPoint - fromPoint) x / 4) asInteger.
	yUnit := ((toPoint - fromPoint) y / 3) asInteger. 

	running := fromPoint.
	path := Array 
		with: (0 @ yUnit) + fromPoint 
		with: ((4 * xUnit) @ ( 2 * yUnit)) + fromPoint 
		with: toPoint .

	^ path collect: [ :p | tmp := LineSegment from: running to: p. running := p. tmp ]! !

MOLineShape subclass: #MOStraightLineShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOStraightLineShape commentStamp: '<historical>' prior: 0!
MOStraightLineShape is the default behavior of an edge. a straight line. Maybe, in the future we will have to do an abstract class for lines, and making this default dehavior a subclass of it.

Class collaborations
	MOEdge MONode MOGraphElement 

Class public API
		#containsPoint:for:	#display:on:

Testing classes
	MOViewRendererTest 

Comments written by AlexandreBergel on 27 April 2011
!

MOStraightLineShape subclass: #MOArrowedLineShape
	instanceVariableNames: 'arrowSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOArrowedLineShape commentStamp: '<historical>' prior: 0!
MOArrowedLineShape represents a straightly line shape with an arrow.

Class collaborations
	MOShapeSelector MOEdge 

Class public API
		#display:on:	#isArrowed

Testing classes
	MOViewRendererTest 

Instance Variables:
	arrowSize --  the size of the arrow, an integer

Comments written by AlexandreBergel on 28 April 2011
!

!MOArrowedLineShape classSide methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 16:57'!
isDefaultLineShape
	"True if I am the default line shape class. I am meant to be overriden. Note that it is assumed to have only one class that answers true"

	^ false! !

!MOArrowedLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:12'!
acceptVisitor: aVisitor
^aVisitor visitArrowedLineShape: self.! !

!MOArrowedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize
	"Return the size of the arrow, an integer"

	^arrowSize! !

!MOArrowedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize: anInteger
	"Set the size of the arrow, an integer"

	arrowSize := anInteger! !

!MOArrowedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 16:58'!
defaultArrowSize
	"Answer the default arrow size"

	^ 6! !

!MOArrowedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 14:27'!
initialize
	"Initialization"

	super initialize.
	self arrowSize: self defaultArrowSize! !

!MOArrowedLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 14:27'!
isArrowed
	"True if I am arrowed. This method is intended to be overriden"

	^ true! !

MOStraightLineShape subclass: #MODashedLineShape
	instanceVariableNames: 'secondaryColor dashLengths'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MODashedLineShape commentStamp: '<historical>' prior: 0!
MODashedLineShape represent a dashed line

Class collaborations
	MOEdge MOEasel MOShapeSelector 

Class public API
		#dashLengths	#dashLengths:	#display:on:	#secondaryColor

Testing classes
	MOViewRendererTest 

Instance Variables:
	secondaryColor --  color of the dash. White per default.
	dashLengths --  Length of the dash. It has to be an integer value

Comments written by AlexandreBergel on 27 April 2011
!

MODashedLineShape subclass: #MOArrowedDashedLineShape
	instanceVariableNames: 'towardsTo arrowSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOArrowedDashedLineShape commentStamp: '<historical>' prior: 0!
MOArrowedDashedLineShape represent an arrowed and dashed line.

Class collaborations
	MOEdge MOShapeSelector 

Class public API
		#display:on:	#isArrowed

Testing classes
	MOViewRendererTest 

Instance Variables:
	towardsTo -- boolean - true if the arrow is on the target node. false the arrow is on the source node
	arrowSize --  Size of the arrow, it has to be an integer.

Comments written by AlexandreBergel on 27 April 2011
!

!MOArrowedDashedLineShape classSide methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 16:57'!
isDefaultLineShape
	"True if I am the default line shape class. I am meant to be overriden. Note that it is assumed to have only one class that answers true"

	^ false! !

!MOArrowedDashedLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:12'!
acceptVisitor: aVisitor
^aVisitor visitArrowedDashedLineShape: self.! !

!MOArrowedDashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize
	"Return the size of the arrow, an integer"

	^arrowSize! !

!MOArrowedDashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 11:20'!
arrowSize: anInteger
	"Set the size of the arrow, an integer"

	arrowSize := anInteger! !

!MOArrowedDashedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 16:58'!
defaultArrowSize
	"Answer the default arrow size"

	^ 6
! !

!MOArrowedDashedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 14:35'!
initialize
	"Constructor"

	super initialize.
	self arrowSize: self defaultArrowSize.
	self towardsTo: true! !

!MOArrowedDashedLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 14:35'!
isArrowed
	"True if I am arrowed. This method is intended to be overriden"

	^ true! !

!MOArrowedDashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/27/2011 12:44'!
towardsTo
	"true if the arrow is on the target node. false the arrow is on the source node"

	^towardsTo! !

!MOArrowedDashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/27/2011 12:45'!
towardsTo: aBoolean
	"true if the arrow is on the target node. false the arrow is on the source node"

	towardsTo:= aBoolean ! !

!MODashedLineShape classSide methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 16:57'!
isDefaultLineShape
	"True if I am the default line shape class. I am meant to be overriden. Note that it is assumed to have only one class that answers true"

	^ false! !

!MODashedLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:13'!
acceptVisitor: aVisitor
^aVisitor visitDashedLineShape: self.! !

!MODashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:30'!
dashLengths
	"Lengths of the two dash. Array of size 2, containing integer. E.g., #(3 3)"
	
	^dashLengths ! !

!MODashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:30'!
dashLengths: twoLengths
	"Set the lengths of the two dash. Array of size 2, containing integer. E.g., #(3 3)"

	dashLengths := twoLengths ! !

!MODashedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 14:31'!
defaultDashLenths
	"Lengths of the two dash. Array of size 2"

	^ #(3 3)! !

!MODashedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 14:31'!
defaultSecondaryDashColor
	"Default color of the second dash"

	^ Color white! !

!MODashedLineShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 14:30'!
initialize
	"Initialization"

	super initialize.
	self dashLengths: self defaultDashLenths.
	self secondaryColor: self defaultSecondaryDashColor! !

!MODashedLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 14:33'!
isDashed
	"True if I am a dashed line. I am meant to be overriden"

	self dashLengths ifNil: [ ^ false ]. "Something strange here, it should not be"
	
	^ self dashLengths second > 0! !

!MODashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:34'!
secondaryColor
	"Return the color of the secondary dash"

	^secondaryColor ! !

!MODashedLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:34'!
secondaryColor: aColor
	"Set the color of the secondary dash"

	secondaryColor:= aColor ! !

!MOStraightLineShape classSide methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 16:57'!
isDefaultLineShape
	"True if I am the default line shape class. I am meant to be overriden. Note that it is assumed to have only one class that answers true"

	^ true! !

!MOStraightLineShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:21'!
acceptVisitor: aVisitor
^aVisitor visitStraightLineShape: self.! !

!MOStraightLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 11:24'!
containsPoint: aPoint for: anEdgeElement
	"True if aPoint is contained within the bounds of anElement graphical representation"

	| attachPoints min max |
	"We do here a special treatment when the line is horizontal or vertical"
	attachPoints := self getAttachPointsFor: anEdgeElement.
	((attachPoints first y = attachPoints second y) and: [ attachPoints first y = aPoint y ])
		ifTrue: [ 
				min := attachPoints first x min: (attachPoints second x). 
				max := attachPoints first x min: (attachPoints second x). 
					^ (min <= aPoint x) and: [ max <= aPoint x] ].

	((attachPoints first x = attachPoints second x) and: [ attachPoints first x = aPoint x ])
		ifTrue: [ 
				min := attachPoints first y min: (attachPoints second y). 
				max := attachPoints first y min: (attachPoints second y). 
					^ (min <= aPoint y) and: [ max <= aPoint y] ].

	^ super containsPoint: aPoint for: anEdgeElement! !

MOShape subclass: #MONodeShape
	instanceVariableNames: 'parent'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MONodeShape commentStamp: '<historical>' prior: 0!
MONodeShape is the root of the shapes to represent nodes. 

MONodeShape is an abstract class. The following methods needs to be implemented


Class collaborations
	MOGraphElement MORoot MOFormsBuilder MOShapeAdaptor MONode MOFormsAdaptor 

Class public API
		#absoluteBoundsFor:	#computeChildrenOriginFor:	#computeOriginRelativeToMeOf:	#computeParentShapeFor:	#containsPoint:for:	#parent	#parent:	#positionOfChildrenForOwner:	#translateBoundsForFigure:by:context:
Testing classes
	MOFormsBuilderTest MOViewRendererTest 

Implementation notes
	

Instance Variables:
	parent --  PLEASE FILL

Comments written by AlexandreBergel on 26 April 2011
!

MONodeShape subclass: #MOFilledShape
	instanceVariableNames: 'fillColor borderColor borderWidth borderStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOFilledShape commentStamp: '<historical>' prior: 0!
MOFilledShape represents a filled shape. A filled shape has a border color, a fill color, a border style, and some facilities for color gradient. Maybe in the future, most of these characteristics will be in decorators.

MOFilledShape is an abstract class. The following methods needs to be implemented


Class collaborations
	MOBrowser DistributionMapExample MOSVGVisitor MOGraphElement MOEasel MOMatrixRenderer MONLinearNormalizer MOAnnouncer MONColorLinearNormalizer MOShapeSelector MOViewRenderer 

Class public API
		#borderColor	#borderColor:	#borderColorFor:	#borderStyle	#borderStyle:	#borderStyleFor:	#borderWidth	#borderWidth:	#borderWidthFor:	#computeBorderColorFor:	#computeBorderStyleFor:	#computeBorderWidthFor:	#computeFillColorFor:	#dashedBorder	#fillColor:	#fillColorFor:	#if:borderColor:	#if:borderWidth:	#if:fillColor:	#simpleBorder
Testing classes
	MondrianExporterSVGTest MOFormsBuilderTest MOViewRendererTest MOLayoutTest MONewFormsBuilderTest MOMatrixRendererTest 

Instance Variables:
	fillColor --  a block or a symbol
	borderColor --  a block or a symbol
	borderWidth --  a block or a symbol
	borderStyle --  a block or a symbol

Comments written by AlexandreBergel on 26 April 2011
!

MOFilledShape subclass: #MOBoundedShape
	instanceVariableNames: 'width height text fontSize fontColor fontFamily logicalFont textAlignment textHorizontalPadding textVerticalPadding'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

MOBoundedShape class
	instanceVariableNames: 'fontCache'!

!MOBoundedShape commentStamp: '<historical>' prior: 0!
MOBoundedShape contains the necessary to manage the width, height and to display a text. 

MOBoundedShape is an abstract class. The following methods needs to be implemented


Class collaborations
	MONFontLinearNormalizer MOMatrixRenderer MONumberLinearNormalizer MOTikzVisitor MORoot MOShapeSelector MOGraphElement MONode MONLinearNormalizer MOEasel MOViewRenderer UMLClassDiagram MOShapeAdaptor MOSVGVisitor 

Class public API
		#bigFont	#computeExtentHavingChildrenFor:	#computeFontColorFor:	#computeFontFamilyFor:	#computeFontSizeFor:	#computeHeightFor:	#computeHeightHavingNoChildrenFor:	#computeTextFor:	#computeTextHeightFor:	#computeTextWidthFor:	#computeWidthFor:	#computeWidthHavingNoChildrenFor:	#defaultBounds	#defaultText	#displaySelection:on:	#extentFor:	#fontColor:	#fontColorFor:	#fontFamily	#fontFamilyFor:	#fontFor:	#fontSize	#fontSize:	#fontSizeFor:	#hasBorder	#height	#height:	#heightFor:	#interlineSpace	#logicalFont:	#normalFont	#scaleElement:by:	#size:	#smallFont	#square:	#squareSize:	#text:	#textAlignment	#textAlignment:	#textFor:	#textHeightFor:	#textHorizontalPadding:	#textPadding:	#textVerticalPadding:	#textWidthFor:	#width	#width:	#widthFor:	#withLabel	#withText	#withoutBackground	#withoutBorder
																																																					
Testing classes
	MOMatrixRendererTest MONewFormsBuilderTest MOViewRendererTest MOCanvasTest MOLayoutTest MondrianExporterSVGTest MOFADELayoutTest MOEventHandlerTest MOElementShapePropertiesTest MOFormsBuilderTest MOSerializationTest 


Instance Variables:
	width --  block, symbol, or value to compute the width. Evaluating the block or performing the symbol on the model should return a number
	height --  block, symbol, or value to compute the height. Evaluating the block or performing the symbol on the model should return a number
	text --  block, symbol, or value to compute the text. Evaluating the block or performing the symbol on the model should return a string
	fontSize --  block, symbol, or valuel to compute the font size. Evaluating the block or performing the symbol on the model should return a number
	fontColor -- block, symbol, or value to compute the font color. Evaluating the block or performing the symbol on the model should return a color
	fontFamily --  block, symbol, or value to compute the width. Evaluating the block or performing the symbol on the model should return a number
	logicalFont --  block, symbol, or value to compute the logical font. Evaluating the block or performing the symbol on the model should return a string (e.g., 'Accuny')
	textAlignment --  #left, #right, #center used to position the text
	textHorizontalPadding --  numerical value for left offset of the text
	textVerticalPadding --  numerical value for above  offset of the text

Class Instance Variables:
	fontCache --  Contains a dictionary. 
	
Comments written by AlexandreBergel on 27 April 2011
!

MOBoundedShape class
	instanceVariableNames: 'fontCache'!

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'Alexandre.Bergel 4/17/2009 13:28'!
defaultColor
	^ Color black! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'Alexandre.Bergel 4/17/2009 13:28'!
defaultFont
	^ TextStyle defaultFont! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'Alexandre.Bergel 3/28/2009 12:28'!
defaultHeight
	^ 5! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'tg 4/26/2010 21:36'!
defaultSize
	^ 5! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'Alexandre.Bergel 3/28/2009 12:28'!
defaultWidth
	^ 5! !

!MOBoundedShape classSide methodsFor: 'instance creation' stamp: 'jlr 10/23/2008 10:04'!
fillColor: aColor 
	^self width: self defaultWidth height: self defaultHeight fillColor: aColor ! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'Alexandre.Bergel 4/17/2009 13:29'!
fontCache

	^ fontCache ifNil: [ fontCache := Dictionary new]! !

!MOBoundedShape classSide methodsFor: 'defaults' stamp: 'AlexandreBergel 10/20/2011 18:03'!
whiteSpacesPerTabulation
	"A tabulation character is equivalent to that amount of white space"

	^ '    '! !

!MOBoundedShape classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 3/28/2009 12:25'!
width: aSmallIntegerOrABlock height: anotherSmallIntegerOrABlock fillColor: aBlock 
	^self new 
		width: aSmallIntegerOrABlock;
		height: anotherSmallIntegerOrABlock;
		fillColor: aBlock! !

!MOBoundedShape classSide methodsFor: 'instance creation' stamp: 'Alexandre.Bergel 2/20/2009 19:03'!
withBorder
	^ self new withBorder! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 14:52'!
bigFont
	"Set the current font as large"

	^ self fontSize: 15! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:44'!
computeExtentHavingChildrenFor: aNodeElement 
	"Answer the width and the height of aNodeElement, taking into account that aNodeElement has subnodes"

	| compoundWidth compoundHeight idealExtentX idealExtentY minX minY bounds |
	"This method is compute the size of the provided element. It therefore has children"
	self assert: [ aNodeElement hasNode ].

	aNodeElement resetMetricCaches.

	compoundWidth := 0.
	compoundHeight := 0.
	minX := 10000000.
	minY := 10000000.

	aNodeElement nodesDo: 
			[ :node | 
				bounds := node bounds.
				compoundWidth := compoundWidth max: (node shape widthFor: node) + bounds origin x.
				compoundHeight := compoundHeight max: (node shape heightFor: node) + bounds origin y.
				minX := minX min: bounds origin x. minY := minY min: bounds origin y].
			
	idealExtentX := self width moValue: aNodeElement model.
	idealExtentY := self height moValue: aNodeElement model.

	^ (compoundWidth + minX max: idealExtentX) @ (compoundHeight + minY max: idealExtentY)! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:44'!
computeFontColorFor: anElement
	"Answer the color of the node element provided as argument The color is computed against the model of the node"

	^ fontColor moValue: anElement model! !

!MOBoundedShape methodsFor: 'convenient' stamp: 'AlexandreBergel 4/29/2011 15:44'!
computeFontFamilyFor: anElement
	"Answer the font family of the node element provided as argument The font family name (a string) is computed against the model of the node"

	^ self fontFamily moValue: anElement model! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:44'!
computeFontSizeFor: anElement
	"Answer the font size of the node element provided as argument The size is computed against the model of the node"

	| textScale |
	textScale := (anElement hasAttribute: #textScale) 
					ifTrue: [ anElement attributeAt: #textScale ]
					ifFalse: [ 1.0 ].

	^ (fontSize moValue: anElement model) * textScale ! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:45'!
computeHeightFor: aNode
	"Answer the height of the node element provided as argument The height, a number, is computed against the model of the node and whether it has subnodes or not"

	^ aNode hasNode
		ifTrue: [ self computeHeightHavingChildrenFor: aNode ]
		ifFalse: [ self computeHeightHavingNoChildrenFor: aNode ]! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:45'!
computeHeightHavingChildrenFor: aNode
	"Answer the height of aNodeElement, taking into account that aNodeElement has subnodes"

	^ (self computeExtentHavingChildrenFor: aNode) y asInteger max: 5! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:45'!
computeHeightHavingNoChildrenFor: aNodeElement 
	"Answer the height of aNodeElement, taking into account that aNodeElement has _no_ subnodes"

 	| v h |
	h := 5.
	text isNil ifFalse: [ h := self textHeightFor: aNodeElement ].

	v := self height moValue: aNodeElement model.
	(v isFloat and: [ v isFinite not ]) ifTrue: [ ^ h ].
	^ v asInteger max: h ! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 10/20/2011 18:01'!
computeTextFor: anElement
	"Answer the text of the node element provided as argument The text, a String, is computed against the model of the node"
	"The tabulation character is replaced by space characters."

	^ (self replaceTabIn: (text moValue: anElement model) asString)! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:45'!
computeTextHeightFor: anElement
	"Answer the text height of the node element provided as argument The text height, a number, is computed against the model of the node"

	| lineCount size textScale |
	lineCount := (self textFor: anElement) lineCount.
	size := (self fontSizeFor: anElement) max: ((self fontFor: anElement) height).
	textScale := (anElement hasAttribute: #textScale) 
					ifTrue: [ anElement attributeAt: #textScale ]
					ifFalse: [ 1.0 ].

	^ (( (size * textScale) asInteger + (self interlineSpace)) * lineCount) + (2 * textVerticalPadding) 
		"+ (2 * (self borderWidthFor: anElement) )"
"	^ ((((self fontFor: anElement) height) + (self interlineSpace))* lineCount) + (2 * textVerticalPadding)"
	
"	^ (((self fontSizeFor: anElement) + (self interlineSpace))* lineCount) + (2 * textVerticalPadding)"! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:45'!
computeTextWidthFor: anElement
	"Answer the width of the text of the node element provided as argument The text width, a number, is computed against the model of the node. Multilines and horizontal padding is taken into account"

	| longestLine |
	longestLine := ''.	
	(self textFor: anElement) linesDo: [:line | 
		longestLine := line size > longestLine size ifTrue: [line] ifFalse: [longestLine]].	
	^ ((self fontFor: anElement) widthOfString: longestLine) + (2 * textHorizontalPadding) 
				+ (2 * (self borderWidthFor: anElement) )! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:46'!
computeWidthFor: aNode
	"Answer the width of the node element provided as argument The width, a number, is computed against the model of the node"

	^ aNode hasNode
		ifTrue: [ self computeWidthHavingChildrenFor: aNode ]
		ifFalse: [ self computeWidthHavingNoChildrenFor: aNode ]! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:46'!
computeWidthHavingChildrenFor: aNode
	"Answer the width of the node element provided as argument The width, a number, is computed against the model of the node and assume that a Node has subnodes"

	^ (self computeExtentHavingChildrenFor: aNode) x asInteger max: 5! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:46'!
computeWidthHavingNoChildrenFor: aNodeElement 
	"Answer the width of the node element provided as argument The width, a number, is computed against the model of the node and assume that a Node has _no_ subnodes"

	| w v |
	w := 5.
	"Note that we do not consider multilines label for now"
	text isNil ifFalse: [ w := self textWidthFor: aNodeElement ].

	v := self width moValue: aNodeElement model.
	(v isFloat and: [ v isFinite not ]) ifTrue: [ ^ w ].
	^ v asInteger max: w ! !

!MOBoundedShape methodsFor: 'default values' stamp: 'AlexandreBergel 4/29/2011 15:46'!
defaultBounds
	"Answer the default bounds"

	^ Rectangle left: 0 top: 0 width: 10 height: 10! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 14:58'!
defaultColor
	"Convenient method for getting the default color. Set in the metaclass actually"

	^ self class defaultColor! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 14:59'!
defaultFont
	"Convenient method for getting the default font. Set in the metaclass actually. Return an instance of a subclass of AbstractFont"

	^ self class defaultFont! !

!MOBoundedShape methodsFor: 'default values' stamp: 'AlexandreBergel 4/29/2011 15:46'!
defaultFontFamily
	"Answer the name (as a string) for the default font family. Could be for example 'Bitmap DejaVu Sans'"

	^ self class defaultFont familyName! !

!MOBoundedShape methodsFor: 'drawing' stamp: 'AlexandreBergel 4/29/2011 15:46'!
defaultSelectionHandlerColor
	"Answer the color of the selection handler"

	^ Color red
! !

!MOBoundedShape methodsFor: 'drawing' stamp: 'AlexandreBergel 4/29/2011 15:46'!
defaultSelectionHandlerSize
	"Answer the size of the selection handler. "

	^ 4
! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:48'!
defaultSize
	"Answer the default size of the nodes"

	^ self class defaultSize! !

!MOBoundedShape methodsFor: 'drawing' stamp: 'AlexandreBergel 4/29/2011 15:02'!
displaySelection: aGraphElement on: aCanvas
	"display the selection of aGraphElement on the canvas. Does not display anything if the element is not displayed"

	| s borderWidthValue c bounds |
	aGraphElement isSelected
		ifFalse: [ ^ self ].
	s := self defaultSelectionHandlerSize.	"Size of handler"
	borderWidthValue := 2.
	c := self defaultSelectionHandlerColor.
	bounds := 0 @ 0 extent: (self extentFor: aGraphElement).
	aCanvas
		line: bounds origin
		to: bounds origin + (s @ 0)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds origin
		to: bounds origin + (0 @ s)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds topRight
		to: bounds topRight - (s @ 0)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds topRight
		to: bounds topRight + (0 @ s)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds bottomRight
		to: bounds bottomRight - (s @ 0)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds bottomRight
		to: bounds bottomRight - (0 @ s)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds bottomLeft
		to: bounds bottomLeft + (s @ 0)
		width: borderWidthValue
		color: c.
	aCanvas
		line: bounds bottomLeft
		to: bounds bottomLeft - (0 @ s)
		width: borderWidthValue
		color: c! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:49'!
extentFor: anElement 
	"Answer the width and the height of the node element"

	^ (self boundsFor: anElement) extent
! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:04'!
fontColor: anObject
	"Set the font color. anObject can be a symbol, a block or a value. anObject is evaluated against the model of the node to be displayed."

	fontColor := anObject! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:49'!
fontColorFor: anElement
	"Answer the font color of the node element provided as argument The color, an instance of the Color class, is computed against the model of the node"

	^ anElement 
		cacheNamed: #cachefontColorFor:
		of: self
		 ifAbsentInitializeWith: [ self computeFontColorFor: anElement ] ! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:49'!
fontFamily
	"Answer the font family, a string"

	^fontFamily! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:05'!
fontFamily: anObject
	"Set the font family. anObject can be a symbol, a block or a value. anObject is evaluated against the model of the node to be displayed. anObject when evaluated, has to return a string"

	fontFamily := anObject! !

!MOBoundedShape methodsFor: 'convenient' stamp: 'AlexandreBergel 4/29/2011 15:06'!
fontFamilyFor: aNodeElement
	"Return the font family for the node aNodeElement"

	^ aNodeElement 
		cacheNamed: #cachefontFamilyFor: 
		of: self
		ifAbsentInitializeWith: [ self computeFontFamilyFor: aNodeElement ]! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:06'!
fontFor: anElement
	"Return the font for the node aNodeElement"

	| size family lf |
	^ self logicalFont isNil 
		ifTrue: [
			size := self fontSizeFor: anElement.
			family := self fontFamilyFor: anElement.
			lf := LogicalFont familyName: family pointSize: size.
			self class fontCache 
				at: (family, size asString) 
				ifAbsentPut: [ lf ] ]
		ifFalse: [self logicalFont moValue: anElement]
	! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:07'!
fontSize
	"Return the font size which can be a symbol, a block or a value. This value is evaluated against the model of the node to be displayed"

	^fontSize! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'Alexandre.Bergel 4/17/2009 13:23'!
fontSize: anObject
	"The size of the font to be used. anObject could be a 1-arg block or a numerical value"

	fontSize := anObject! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:53'!
fontSizeFor: anElement
	"Answer the font size for the node aNodeElement"

	^ anElement 
		cacheNamed: #cachefontSizeFor: 
		of: self
		ifAbsentInitializeWith: [ self computeFontSizeFor: anElement ]
	! !

!MOBoundedShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 15:56'!
hasBorder
	"True if I have a border. This method makes no sense. It has to be removed"

	^ borderWidth = 1! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:58'!
height
	"Answer the height which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^height! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:58'!
height: aBlock
	"Set the height which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	height := aBlock! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:09'!
heightFor: anElement
	"Return the height of the node element provided as argument The height, a number, is computed against the model of the node"

	^ anElement 
		cacheNamed: #cacheheightFor: 
		of: self
		ifAbsentInitializeWith: [ self computeHeightFor: anElement ]
	! !

!MOBoundedShape methodsFor: 'initialize-release' stamp: 'AlexandreBergel 4/29/2011 15:10'!
initialize
	"Initialization"

	super initialize.
	width := self defaultSize.
	height := self defaultSize.
	borderWidth := 1.
	text := nil.
	self fontFamily: self defaultFontFamily.
	self fontSize: self defaultSize.
	self fontColor: self defaultColor.
	textAlignment := #center.
	textHorizontalPadding := textVerticalPadding := 0 
! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:10'!
interlineSpace
	"Space between lines in a multi-line text"

	^ 3! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:07'!
logicalFont
	"Return the logical font which can be a symbol, a block or a value. This value is evaluated against the model of the node to be displayed"

	^ logicalFont! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:07'!
logicalFont: anObject
	"Set the logical font which can be a symbol, a block or a value. This value is evaluated against the model of the node to be displayed"

	logicalFont := anObject! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:12'!
normalFont
	"Set the current font as normal"

	^ self fontSize: 10! !

!MOBoundedShape methodsFor: 'text manipulation' stamp: 'AlexandreBergel 10/20/2011 18:29'!
replaceTabIn: aByteString 
	"Replace all tab character with white spaces in aByteString"

	| stream |
	stream := WriteStream on: String new. 
	aByteString do: [ :c | 
		c = Character tab 
			ifTrue: [ stream nextPutAll: self class whiteSpacesPerTabulation ] 
			ifFalse: [ stream nextPut: c ] ].
	^ stream contents! !

!MOBoundedShape methodsFor: 'scaling' stamp: 'AlexandreBergel 4/28/2011 23:40'!
scaleElement: anElement by: aNumber
	"Scale an element by a value. Accomodate the font size. However this is not perfect for now."

	"We scale the bounds"
	super scaleElement: anElement by: aNumber.
	
	"We check if there is a text. If this is the case, we caches stored in the attribute has to be updated if they exist"
	text ifNotNil: [
		anElement attributeAt: #textScale ifAbsentPut: [ 1 ].

		#(#cachetextWidthFor: #cachefontSizeFor: #cachetextHeightFor: #textScale) do: [ :s | 
			(anElement hasCacheNamed: s of: self) 
					ifTrue: [ anElement 
						cacheNamed: s
						of: self
						put: (anElement cacheNamed: s of: self) * (((aNumber - 1) * 0.5) + 1) ] ] ]! !

!MOBoundedShape methodsFor: 'convenient' stamp: 'AlexandreBergel 4/29/2011 15:13'!
size: aValueOrABlockOrASymbol
	"Set both the width and the height. aValueOrABlockOrASymbol is evaluated against the model of the node to be displayed"

	self width: aValueOrABlockOrASymbol.
	self height: aValueOrABlockOrASymbol.
	! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:13'!
smallFont
	"Set the current font to small"

	^ self fontSize: 5! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:18'!
text: anObject
	"Set the text which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	text := anObject! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:08'!
textAlignment 
	"Return the text alignment, a symbol. #left, #right, #center used to position the text"

	^ textAlignment! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:09'!
textAlignment: aSymbol
	"Set the text alignment, a symbol. #left, #right, #center used to position the text"

	textAlignment := aSymbol! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:15'!
textFor: anElement
	"Return the text of the node element provided as argument The text, an instance of String, is computed against the model of the node"
	
	^ anElement 
		cacheNamed: #cachetextFor: 
		of: self
		ifAbsentInitializeWith: [ self computeTextFor: anElement ]
! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:15'!
textHeightFor: anElement
	"Return the text height of the node element provided as argument The height, a number, is computed against the model of the node"

	^ anElement 
		cacheNamed: #cachetextHeightFor: 
		of: self
		ifAbsentInitializeWith: [ self computeTextHeightFor: anElement ]
! !

!MOBoundedShape methodsFor: 'text padding' stamp: 'AlexandreBergel 6/19/2011 12:24'!
textHorizontalPadding
	^ textHorizontalPadding.! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:17'!
textHorizontalPadding: anInteger
	"Set the horizontal padding with an integer. This is the space in pixels between the left of the shape and the beginning of the text"

	textHorizontalPadding := anInteger ! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:16'!
textPadding: anInteger
	"set both the horizontal and vertical padding with an integer"

	textHorizontalPadding := textVerticalPadding := anInteger ! !

!MOBoundedShape methodsFor: 'text padding' stamp: 'AlexandreBergel 6/19/2011 12:24'!
textVerticalPadding
	^ textVerticalPadding.! !

!MOBoundedShape methodsFor: 'accessing text' stamp: 'AlexandreBergel 4/29/2011 15:17'!
textVerticalPadding: anInteger
	"Set the vertical padding using an integer. This is the space in pixels between the top of the shape and the beginning of the text"

	textVerticalPadding := anInteger ! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:17'!
textWidthFor: anElement
	"Return the text width of the node element provided as argument The width, a number, is computed against the model of the node"

	^ anElement 
		cacheNamed: #cachetextWidthFor:
		of: self
		ifAbsentInitializeWith: [ self computeTextWidthFor: anElement ]
! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:20'!
width
	"Return the width which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^width! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:20'!
width: anObject
	"Set the width which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	width := anObject! !

!MOBoundedShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 15:20'!
widthFor: aNodeElement
	"Return the shape width of the node element provided as argument The width, a number, is computed against the model of the node"

	^ aNodeElement 
		cacheNamed: #cachewidthFor: 
		of: self
		ifAbsentInitializeWith: [ self computeWidthFor: aNodeElement ]
! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:19'!
withBorder
	"Set the shape as a bordered shape, with a width of 1"

	self borderWidth: 1.! !

!MOBoundedShape methodsFor: 'convenient' stamp: 'AlexandreBergel 4/29/2011 15:19'!
withText
	"Set the bounded shape as a labeled one"

	self text: #yourself! !

!MOBoundedShape methodsFor: 'convenient' stamp: 'AlexandreBergel 4/29/2011 15:19'!
withoutBackground
	"Remove the background of the shape"

	fillColor := nil! !

!MOBoundedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:18'!
withoutBorder
	"Remove the border of the shape"

	self borderWidth: 0.! !

MOBoundedShape subclass: #MOChildrenShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOChildrenShape commentStamp: '<historical>' prior: 0!
MOChildrenShape is essentially used as a marker in a forms builder.  This point has been discussed "MOChildrenShape has some flavor of marker in the way it is used and implemented. Is it really a shape or does it indicate a location in the grid where the subnodes have to be displayed?" It is not clear whether MOChildrenShape should solely be a marker or not.

Class collaborations
	MOFormsBuilder MOShapeAdaptor MOGraphElement MOFormsShape 

Class public API
		#display:on:	#extentFor:	#isChildShape	#originForChildrenElement:

Testing classes
	MOFormsBuilderTest 

Comments written by AlexandreBergel on 27 April 2011
!

!MOChildrenShape methodsFor: 'drawing' stamp: 'AlexandreBergel 8/2/2011 17:32'!
absoluteOriginForChildrenElement: anElement
	"Return the top-left position where the child has to be drawn"
	"Return the absolute position origin"
	
	^ (anElement shape builder boundsOf: self for: anElement) origin + anElement absoluteBounds origin! !

!MOChildrenShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:13'!
acceptVisitor: aVisitor
^aVisitor visitChildrenShape: self.! !

!MOChildrenShape methodsFor: 'drawing' stamp: 'AlexandreBergel 4/29/2011 16:41'!
extentFor: aNodeElement 
	"Answer the extent (width and height) of the node element"
	
	^ ( self computeBoundsFor: aNodeElement) extent
! !

!MOChildrenShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/29/2011 16:41'!
isChildShape
	"True if I am a child shape. This method is intended to be overriden."

	^ true ! !

MOBoundedShape subclass: #MODiamondShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MODiamondShape commentStamp: '<historical>' prior: 0!
MODiamondShape is a diamond shape

Class collaborations
	MONode 

Class public API
		#display:on:

Comments written by AlexandreBergel on 27 April 2011
!

!MODiamondShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:14'!
acceptVisitor: aVisitor
^aVisitor visitDiamondShape: self.! !

MOBoundedShape subclass: #MOEllipseShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOEllipseShape commentStamp: '<historical>' prior: 0!
MOEllipseShape represents an ellipse shape

Class collaborations
	MONode 

Class public API
		#display:on:

Testing classes
	 MOViewRendererTest

Comments written by AlexandreBergel on 27 April 2011
!

!MOEllipseShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:14'!
acceptVisitor: aVisitor
^aVisitor visitEllipseShape: self.! !

MOBoundedShape subclass: #MOImageShape
	instanceVariableNames: 'form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOImageShape commentStamp: '<historical>' prior: 0!
MOImageShape represents a bitmap as a shape

Class collaborations
	MONode MondrianIcons MOEasel 

Class public API
		#display:on:	#form	#form:	#heightFor:	#widthFor:

Testing classes
	MOViewRendererTest 

Instance Variables:
	form --  Instance of Form, the bitmap

Comments written by AlexandreBergel on 27 April 2011
!

!MOImageShape classSide methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/21/2010 13:48'!
form: aForm
	^ self new form: aForm! !

!MOImageShape methodsFor: 'visitor' stamp: 'AlexandreBergel 6/26/2011 17:20'!
acceptVisitor: aVisitor
	^ aVisitor visitImageShape: self.! !

!MOImageShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:45'!
form
	"Answer the bitmap of the shape"

	^ form ifNil: [ form := MondrianIcons mondrian ]! !

!MOImageShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:46'!
form: anObject
	"Set the bitmap of the shape. anObject could be a block, a symbol or a simple value. It is then evaluated against the model of the node to render"

	form := anObject! !

!MOImageShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:48'!
heightFor: aNode
	"Answer the height of the node. It is equal to the height of the bitmap"

	^ self form height! !

!MOImageShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:49'!
widthFor: aNode
	"Answer the width of the node. It is equal to the width of the bitmap"

	^ self form width! !

MOBoundedShape subclass: #MORectangleShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MORectangleShape commentStamp: '<historical>' prior: 0!
MORectangleShape is a Shape that describes a rectangle. A refactoring started to times ago to move some of the features of Bounded Shape into this class. But it hasn't finished.

Class collaborations
	MONode MOFormsShape CountingNullCanvas 

Class public API
		#display:on:
	
Comments written by AlexandreBergel on 27 April 2011
!

MORectangleShape subclass: #MOFixedRectangleShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOFixedRectangleShape commentStamp: '<historical>' prior: 0!
MOFixedRectangleShape is a rectangle with a fixed size. MORectangleShape has a variable size, depending on its children. A fixed rectangle has a fixed size, no matter its children.

Class collaborations
	MOGraphElement 

Class public API
		#computeHeightFor:	#computeWidthFor:

Testing classes
	MOViewRendererTest

Comments written by AlexandreBergel on 28 April 2011
!

!MOFixedRectangleShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 16:50'!
computeHeightFor: aNode
	"Answer the height of the shape for the node provided as argument."

	| h |
	h := 5.
	^ (self height moValue: aNode model) asInteger max: h! !

!MOFixedRectangleShape methodsFor: 'public' stamp: 'AlexandreBergel 4/29/2011 16:50'!
computeWidthFor: aNode
	"Answer the width of the shape for the node provided as argument."

	| w |
	w := 5.
	^ (self width moValue: aNode model) asInteger max: w! !

!MORectangleShape classSide methodsFor: 'testing' stamp: 'Alexandre Bergel 6/17/2010 12:10'!
isDefaultNodeShape
	^ true! !

!MORectangleShape methodsFor: 'visitor' stamp: 'AlexandreBergel 10/20/2011 17:37'!
acceptVisitor: aVisitor
	^ aVisitor visitRectangleShape: self.! !

MOBoundedShape subclass: #MORoundedRectangleShape
	instanceVariableNames: 'radius'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MORoundedRectangleShape commentStamp: '<historical>' prior: 0!
MORoundedRectangleShape is a rectangle with rounded corners

Class collaborations
	MONode MOShapeSelector 

Class public API
		#display:on:

Testing classes
	MOViewRendererTest 

Instance Variables:
	radius -- radius of the rounded corner.

Comments written by AlexandreBergel on 27 April 2011
!

!MORoundedRectangleShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:19'!
acceptVisitor: aVisitor
^aVisitor visitRoundedRectangleShape: self.! !

!MORoundedRectangleShape methodsFor: 'initialization' stamp: 'AlexandreBergel 4/29/2011 16:53'!
initialize
	"initialization"

	super initialize.
	self radius: 10! !

!MORoundedRectangleShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:54'!
radius
	"Answer the radius of the rectangle. An integer."

	^ radius! !

!MORoundedRectangleShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 16:54'!
radius: integer
	"Set the radius of the rectangle. An integer."

	radius := integer! !

MOBoundedShape subclass: #MOTriangleShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOTriangleShape commentStamp: '<historical>' prior: 0!
MOTriangleShape represents a triangle. 

Class collaborations
	MONode 

Class public API
		#display:on:

Testing classes
	MOViewRendererTest

Comments written by AlexandreBergel on 27 April 2011
!

!MOTriangleShape methodsFor: 'visitor' stamp: 'SantiagoVidal 5/18/2011 09:21'!
acceptVisitor: aVisitor
^aVisitor visitTriangleShape: self.! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:36'!
borderColor
	"Answer the border color which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^ borderColor! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:37'!
borderColor: anObject
	"Set the border color which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	borderColor := anObject! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
borderColorFor: anElement
	"Answer the border color corresponding of the node element provided as parameter. The result is eventually cached"

	^ anElement 
		cacheNamed: #cacheborderColorFor:
		of: self
		ifAbsentInitializeWith: [ self computeBorderColorFor: anElement ]
! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:37'!
borderStyle 
	"Set the border style which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^ borderStyle! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:38'!
borderStyle: anObject
	"Set the border style which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed. When evaluated, anObject has to return a instance of BorderStyle. Check the class BorderStyle for more information"

	borderStyle := anObject! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
borderStyleFor: anElement
	"Answer the border style corresponding of the node element provided as parameter. The result is eventually cached"

	^ anElement 
		cacheNamed: #cacheborderStyleFor:
		of: self
		ifAbsentInitializeWith: [ self computeBorderStyleFor: anElement ]
! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:38'!
borderWidth
	"Answer the border widht which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^ borderWidth! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:39'!
borderWidth: anObject
	"Answer the border widht which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	borderWidth := anObject! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
borderWidthFor: anElement
	"Answer the border width corresponding of the node element provided as parameter. The result is eventually cached"

	^ anElement 
		cacheNamed: #cacheborderWidthFor:
		of: self
		ifAbsentInitializeWith: [ self computeBorderWidthFor: anElement ]
! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
computeBorderColorFor: anElement
	"Answer the border color against the model of the provided node element"

	^ borderColor moValue: anElement model! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:30'!
computeBorderStyleFor: anElement
	"Answer the border style against the model of the provided node element"

	^ borderStyle moValue: anElement model! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:30'!
computeBorderWidthFor: anElement
	"Answer the border width against the model of the provided node element"

	^ borderWidth moValue: anElement model! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
computeFillColorFor: anElement
	"Answer the fill color of the element, computed from the model of the element"

	^ fillColor moValue: anElement model! !

!MOFilledShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 15:32'!
dashedBorder
	"Set the border of the shape as dashed"

	self borderStyle: BorderStyle dashed! !

!MOFilledShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 15:33'!
defaultBorderColor
	"Answer the default border color"

	^ Color black! !

!MOFilledShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 15:33'!
defaultBorderWidth
	"Answer the default border width"

	^ 1! !

!MOFilledShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 15:34'!
defaultFillColor
	"Answer the default fill color of the shape"

	^ Color white! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:36'!
fillColor
	"Answer the fill color which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed"

	^ fillColor! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 15:36'!
fillColor: anObjectOrABlock
	"Set the fill color which can be a symbol, a block or a value. This object is evaluated against the model of the node to be displayed. Any object may be provided to this method. It could be: 
		- a color (e.g., 'Color blue')
		- a block (e.g., [:each | each size > 10 ifTrue: [Color blue] ifFalse: [Color red]]
		- an instance of MONColorLinearNormalizer (e.g., linearFillColor:within:)"

	fillColor := anObjectOrABlock! !

!MOFilledShape methodsFor: 'private' stamp: 'AlexandreBergel 4/29/2011 15:31'!
fillColorFor: anElement
	"Answer the color corresponding of the node element provided as parameter. The result is eventually cached"

	^ anElement 
		cacheNamed: #cachefillColorFor:
		of: self
		ifAbsentInitializeWith: [ self computeFillColorFor: anElement ]
! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:43'!
if: conditionBlock borderColor: colorBlock
	"If conditionBlock is evaluated at true, then colorBlock is used to pick the color to use. Both conditionBlock and colorBlock are evaluated with the model value of the node."

	| oldBlockOrValue |
	oldBlockOrValue := self borderColor ifNil: [ self defaultBorderColor ].
	^self borderColor: [ :each | (conditionBlock moValue: each)
								ifTrue: [ colorBlock moValue: each ] ifFalse: [ oldBlockOrValue moValue: each ]].! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:43'!
if: conditionBlock borderWidth: widthBlock
	"If conditionBlock is evaluated at true, then widthBlock is used to set the width of the border. Both conditionBlock and widthBlock are evaluated with the model value of the node."

	| oldBlockOrValue |
	oldBlockOrValue := self borderWidth ifNil: [ self defaultBorderWidth ].
	^self borderWidth: [ :each | (conditionBlock moValue: each)
								ifTrue: [ widthBlock moValue: each ] ifFalse: [ oldBlockOrValue moValue: each ]].! !

!MOFilledShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/29/2011 14:43'!
if: conditionBlock fillColor: colorBlock
	"If conditionBlock is evaluated at true, then colorBlock is used to set the color of the node. Both conditionBlock and colorBlock are evaluated with the model value of the node."

	| oldBlockOrValue |
	oldBlockOrValue := self fillColor ifNil: [ self defaultFillColor ].
	^self fillColor: [ :each | (conditionBlock moValue: each)
								ifTrue: [ colorBlock moValue: each ] ifFalse: [ oldBlockOrValue moValue: each ]].! !

!MOFilledShape methodsFor: 'initialize' stamp: 'AlexandreBergel 4/29/2011 15:31'!
initialize
	"Initialization"

	super initialize.
	self fillColor: self defaultFillColor.
	self borderColor: self defaultBorderColor.
	self borderWidth: self defaultBorderWidth! !

!MOFilledShape methodsFor: 'configuration' stamp: 'AlexandreBergel 4/29/2011 15:34'!
simpleBorder
	"Make the shape use a simple border style"

	self borderStyle: BorderStyle simple! !

!MONodeShape classSide methodsFor: 'as yet unclassified' stamp: 'Alexandre Bergel 6/17/2010 12:07'!
default
	^ Default ifNil: [ Default := MORectangleShape new ]! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 9/19/2010 16:56'!
absoluteBoundsFor: aGraphElement
	"Answer the bounds in absolute terms (relative to the entire Canvas, not just the parent)."

	| b |
	b :=  self boundsFor: aGraphElement.
	^ aGraphElement owner isNil
		ifTrue: [ b ]
		ifFalse: [ b translateBy: (self positionOfChildrenForOwner: aGraphElement) ]
! !

!MONodeShape methodsFor: 'accessing' stamp: 'AlexandreBergel 4/26/2011 17:50'!
computeChildrenOriginFor: aGraphElement
	"Return the origin of the graph element according to the position of the children shape."
	| builder childrenShape ownerPosition elementOrigin deltaX deltaY childrenPosition |

	self 
		assert: [ aGraphElement owner notNil ] 
		description: 'Not an inner node'.	
	self 
		assert: [ aGraphElement owner shape isFormsShape ] 
		description: 'The shape of the owner is not a form shape'.

	builder := aGraphElement getAssociatedBuilder.
	childrenShape := aGraphElement computeParentShape.

	ownerPosition := builder boundsOf: childrenShape for: aGraphElement owner.
	^ ownerPosition origin.
	
	
	"elementOrigin := (builder boundsOf: childrenShape for: aGraphElement)."
"	elementOrigin := aGraphElement bounds origin.

	childrenPosition := builder positionOf:  childrenShape. 

	deltaX := ((builder columnAlignmentAt: childrenPosition x) ~= #fill)
					ifTrue: [ ownerPosition origin x - elementOrigin x ]
					ifFalse: [ 0 ].
	deltaY := ((builder rowAlignmentAt: childrenPosition y) ~= #fill)
					ifTrue: [ ownerPosition origin y - elementOrigin y ]
					ifFalse: [ 0 ].

	^ elementOrigin translateBy: (deltaX @ deltaY)  "
! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 5/4/2011 14:35'!
computeExtentFor: anElement 
	"Compute the extent of an element"

	^ (self widthFor: anElement) @ (self heightFor: anElement)! !

!MONodeShape methodsFor: 'testing' stamp: 'AlexandreBergel 4/9/2011 11:54'!
containsPoint: aPoint for: anElement
	"True if the absolute bounds of the element contains aPoint"

	^ anElement absoluteBounds containsPoint: aPoint! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 5/4/2011 14:35'!
heightFor: element
	self subclassResponsibility ! !

!MONodeShape methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/13/2010 14:53'!
parent
	^parent! !

!MONodeShape methodsFor: 'accessing' stamp: 'Alexandre Bergel 3/13/2010 14:53'!
parent: aParentShape
	parent := aParentShape! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 4/26/2011 17:51'!
positionOfChildrenForOwner: aGraphElement
	| ownerAbsolutePosition |
	ownerAbsolutePosition := aGraphElement owner absoluteBounds origin.
	^ aGraphElement hasParentShape
		ifTrue: [ (aGraphElement computeChildrenOrigin) + ownerAbsolutePosition ]
		ifFalse: [ ownerAbsolutePosition ]! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 4/18/2011 17:41'!
translateBoundsForFigure: aGraphElement by: aDelta context: aDictionary
	"Move the shape"
	
	aDelta = (0 @ 0) ifTrue: [^self].
	aGraphElement translateBy: aDelta.
	self changed! !

!MONodeShape methodsFor: 'public' stamp: 'AlexandreBergel 5/4/2011 14:35'!
widthFor: element
	self subclassResponsibility ! !

!MOViewRenderer methodsFor: '*Mondrian-Shapes' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
shape
	^ MOShapeSelector new view: self! !

Object subclass: #MOShapeSelector
	instanceVariableNames: 'view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Shapes'!

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'VeronicaUquillas 4/8/2010 10:16'!
arrowedDashedLine
	^ view shape: MOArrowedDashedLineShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
arrowedLine
	^ view shape: MOArrowedLineShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
curvedLine
	^ view shape: MOCurveFitterShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'VeronicaUquillas 4/2/2010 15:34'!
dashedLine
	^ view shape: MODashedLineShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre Bergel 4/1/2010 15:51'!
diamond
	^ view shape: MODiamondShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre Bergel 12/17/2009 02:31'!
ellipse
	^ view shape: MOEllipseShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'tg 12/9/2009 12:50'!
horizontalOrthogonalLine
	^ view shape: MOOrthoHorizontalLineShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'AlexandreBergel 12/21/2010 13:59'!
image
	^ view shape: (MOImageShape new)! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre Bergel 12/22/2009 00:37'!
label
	^ view shape: (MORectangleShape new withoutBorder; withText)! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
line
	^ view shape: MOStraightLineShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
rectangle
	^ view shape: MORectangleShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre Bergel 3/10/2010 12:07'!
roundedRectangle
	^ view shape: MORoundedRectangleShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'Alexandre.Bergel 5/14/2009 00:44'!
splineLine
	^ view shape: MOSplineShape new! !

!MOShapeSelector methodsFor: 'basic shape node' stamp: 'Alexandre Bergel 4/1/2010 15:48'!
triangle
	^ view shape: MOTriangleShape new! !

!MOShapeSelector methodsFor: 'basic shape edge' stamp: 'tg 10/31/2009 11:00'!
verticalOrthogonalLine
	^ view shape: MOOrthoVerticalLineShape new! !

!MOShapeSelector methodsFor: 'accessing' stamp: 'Alexandre.Bergel 5/14/2009 00:45'!
view: aView
	view := aView! !
